@@ -26,42 +26,54 @@
 #include "extern.h"
 
 void
-anubis_error(int method, char *format, ...)
+anubis_error(int method, const char *fmt, ...)
 {
 	va_list arglist;
-	char txt[LINEBUFFER+1];
 
-	va_start(arglist, format);
-	vsnprintf(txt, LINEBUFFER, format, arglist);
-	va_end(arglist);
-
-	if (method == HARD)
+	switch (method) {
+	case HARD:
 		topt |= T_ERROR;
-	else if (method == SOFT)
+		break;
+	case SOFT:
+	case SYNTAX:
 		topt &= ~T_ERROR;
+		break;
+	}
 
-	if (options.termlevel != SILENT) {
+	if (topt & T_SMTP_ERROR_CODES) {
+		fprintf(stdout, "451 4.0.0 ");
+		va_start(arglist, fmt);
+		vfprintf(stdout, fmt, arglist);
+		va_end(arglist);
+		fputc('\n', stdout);
+		
+	}
+	else if (options.termlevel != SILENT) {
+		char msg[LINEBUFFER+1];
+	
+		va_start(arglist, fmt);
+		vsnprintf(msg, LINEBUFFER, fmt, arglist);
+		va_end(arglist);
 #ifdef HAVE_SYSLOG
 		if ((topt & T_DAEMON) && !(topt & T_FOREGROUND)) {
-			if (options.slogfile)
-				filelog(options.slogfile, txt);
-			else
-				syslog(LOG_ERR | LOG_MAIL, txt);
-
+			syslog(LOG_ERR | LOG_MAIL, msg);
 			if (options.ulogfile && options.uloglevel >= FAILS)
-				filelog(options.ulogfile, txt);
+				filelog(options.ulogfile, msg);
 		}
 		else
 #endif /* HAVE_SYSLOG */
 			if (topt & T_FOREGROUND)
-				mprintf(">>[%d] %s", (int)getpid(), txt);
+				mprintf("%s[%d] %s",
+					method == SYNTAX ? "" : ">>",
+					(int)getpid(), msg);
 			else
-				mprintf(">>%s", txt);
+				mprintf("%s%s",
+					method == SYNTAX ? "" : ">>",
+					msg);
 	}
 	errno = 0;
-	if (!(topt & T_DAEMON) && !(topt & T_FOREGROUND))
+	if (method != SYNTAX && !(topt & T_DAEMON) && !(topt & T_FOREGROUND))
 		quit(EXIT_FAILURE);
-	return;
 }
 
 void
@@ -69,14 +81,12 @@ socket_error(const char *msg)
 {
 	anubis_error(HARD, _("Couldn't write to socket: %s."),
 		     msg ? msg : strerror(errno));
-	return;
 }
 
 void
-socks_error(char *txt)
+socks_error(const char *msg)
 {
-	anubis_error(HARD, _("SOCKS proxy: %s"), txt);
-	return;
+	anubis_error(HARD, _("SOCKS proxy: %s"), msg);
 }
 
 void
