@@ -24,63 +24,56 @@
 
 #include "headers.h"
 #include "extern.h"
+#include "rcfile.h"
 
 void
-mprintf(char *format, ...)
+mprintf(const char *fmt, ...)
 {
 	va_list arglist;
-	char txt[LINEBUFFER+1];
 
 	if (options.termlevel == SILENT)
 		return;
 
-	va_start(arglist, format);
-	vsnprintf(txt, LINEBUFFER, format, arglist);
+	va_start(arglist, fmt);
+	vfprintf(stderr, fmt, arglist);
 	va_end(arglist);
-
-	puts(txt);
-	return;
+	fputc('\n', stderr);
 }
 
 void
-info(int mode, char *format, ...)
+info(int mode, const char *fmt, ...)
 {
 	va_list arglist;
-	char txt[LINEBUFFER+1];
+	char msg[LINEBUFFER+1];
 
 	if (mode > options.termlevel)
 		return;
 
-	va_start(arglist, format);
-	vsnprintf(txt, LINEBUFFER, format, arglist);
+	va_start(arglist, fmt);
+	vsnprintf(msg, LINEBUFFER, fmt, arglist);
 	va_end(arglist);
 
 #ifdef HAVE_SYSLOG
 	if ((topt & T_DAEMON) && !(topt & T_FOREGROUND)) {
-		if (options.slogfile)
-			filelog(options.slogfile, txt);
-		else
-			syslog(LOG_INFO | LOG_MAIL, txt);
-
+		syslog(LOG_INFO | LOG_MAIL, msg);
 		if (options.ulogfile && options.uloglevel >= ALL)
-			filelog(options.ulogfile, txt);
+			filelog(options.ulogfile, msg);
 	}
 	else
 #endif /* HAVE_SYSLOG */
 		if (topt & T_FOREGROUND)
-			mprintf("> [%d] %s", (int)getpid(), txt);
+			mprintf("> [%d] %s", (int)getpid(), msg);
 		else
-			mprintf("> %s", txt);
-	return;
+			mprintf("> %s", msg);
 }
 
 void
-filelog(char *logfile, char *txt)
+filelog(char *logfile, char *msg)
 {
 	FILE *fplog;
 
 	fplog = fopen(logfile, "a");
-	if (fplog == 0)
+	if (fplog == NULL)
 		return;
 	else {
 		time_t tp;
@@ -92,10 +85,42 @@ filelog(char *logfile, char *txt)
 		timeptr = localtime(&tp);
 		strftime(timebuf, sizeof(timebuf) - 1,
 			"%a, %d %b %Y %H:%M:%S", timeptr);
-		fprintf(fplog, "%s [%d] %s\n", timebuf, (int)getpid(), txt);
+		fprintf(fplog, "%s [%d] %s\n", timebuf, (int)getpid(), msg);
 		fclose(fplog);
 	}
-	return;
+}
+
+void
+tracefile(RC_LOC *loc, const char *fmt, ...)
+{
+	va_list ap;
+	int n = 0;
+	char msg[LINEBUFFER+1];
+
+	if (!(topt & (T_TRACEFILE_SYS | T_TRACEFILE_USR)))
+		return;
+
+	if (loc)
+		n = snprintf(msg, LINEBUFFER, "%s:%lu: ",
+			     loc->file, (unsigned long)loc->line);
+	va_start(ap, fmt);
+	vsnprintf(msg + n, LINEBUFFER - n, fmt, ap);
+	va_end(ap);
+
+	if ((topt & T_TRACEFILE_SYS) && options.termlevel != SILENT) {
+#ifdef HAVE_SYSLOG
+		if ((topt & T_DAEMON) && !(topt & T_FOREGROUND))
+			syslog(LOG_INFO | LOG_MAIL, msg);
+		else
+#endif /* HAVE_SYSLOG */
+			if (topt & T_FOREGROUND)
+				mprintf("> [%d] %s", (int)getpid(), msg);
+			else
+				mprintf("> %s", msg);
+	}
+
+	if (topt & T_TRACEFILE_USR)
+		filelog(options.tracefile, msg);
 }
 
 /* EOF */
