@@ -25,14 +25,14 @@
 #include "headers.h"
 #include "extern.h"
 
-void
+RETSIGTYPE
 sig_exit(int code)
 {
 	info(NORMAL, _("Signal Caught. Exiting Cleanly..."));
 	quit(code);
 }
 
-void
+RETSIGTYPE
 sig_timeout(int code)
 {
 	info(NORMAL, _("Timeout! Exiting..."));
@@ -52,8 +52,8 @@ free_mem(void)
 	gpg_free();
 #endif /* HAVE_GPG */
 
-	xfree(options.slogfile);
 	xfree(options.ulogfile);
+	xfree(options.tracefile);
 	xfree(session.execpath);
 	xfree_pptr(session.execargs);
 }
