@@ -202,13 +202,13 @@ static const short rc_yyrhs[] = {    31,
 
 #if YYDEBUG != 0
 static const short rc_yyrline[] = { 0,
-   112,   117,   121,   132,   141,   145,   149,   155,   155,   163,
-   173,   176,   180,   193,   197,   198,   199,   200,   201,   202,
-   211,   229,   235,   240,   247,   250,   257,   266,   267,   271,
-   278,   285,   294,   295,   298,   304,   312,   317,   320,   325,
-   328,   336,   346,   353,   357,   363,   373,   379,   387,   391,
-   394,   400,   407,   410,   413,   421,   431,   438,   439,   442,
-   453,   464,   475,   486,   497,   513
+   112,   117,   121,   132,   143,   147,   151,   157,   157,   165,
+   175,   178,   182,   195,   199,   200,   201,   202,   203,   204,
+   213,   231,   237,   242,   249,   252,   259,   268,   269,   273,
+   280,   287,   296,   297,   300,   306,   314,   319,   322,   327,
+   330,   338,   348,   355,   359,   365,   375,   381,   389,   393,
+   396,   402,   409,   412,   415,   423,   433,   440,   441,   444,
+   455,   466,   477,   488,   499,   515
 };
 #endif
 
@@ -904,35 +904,37 @@ case 4:
 #line 133 "rcfile.y"
 {
 		   lex_clear_state();
-		   error_sync_begin();
-		   rc_yyerrok;
-		   rc_yyclearin;
+		   rc_yychar = error_sync_begin();
+		   if (rc_yychar > 0) {
+			   rc_yyerrok;
+			   rc_yyclearin;
+		   }
 	   ;
     break;}
 case 5:
-#line 142 "rcfile.y"
+#line 144 "rcfile.y"
 {
 		   rc_yyval.section = NULL;
 	   ;
     break;}
 case 6:
-#line 146 "rcfile.y"
+#line 148 "rcfile.y"
 {
 		   rc_yyval.section = rc_section_create(rc_yyvsp[-2].begin_sec.name, rc_yyvsp[-2].begin_sec.line, rc_yyvsp[-1].stmtlist.head);
 	   ;
     break;}
 case 7:
-#line 150 "rcfile.y"
+#line 152 "rcfile.y"
 {
 		   rc_yyval.section = NULL;
 	   ;
     break;}
 case 8:
-#line 155 "rcfile.y"
+#line 157 "rcfile.y"
 { verbatim(); ;
     break;}
 case 9:
-#line 156 "rcfile.y"
+#line 158 "rcfile.y"
 {
 		   rc_yyval.begin_sec.line = cfg_line_num - 1;
 		   rc_yyval.begin_sec.name = rc_yyvsp[-1].string;
@@ -942,7 +944,7 @@ case 9:
 	   ;
     break;}
 case 10:
-#line 164 "rcfile.y"
+#line 166 "rcfile.y"
 {
 		   rc_yyval.begin_sec.line = cfg_line_num - 1;
 		   rc_yyval.begin_sec.name = rc_yyvsp[-1].string;
@@ -952,13 +954,13 @@ case 10:
 	   ;
     break;}
 case 12:
-#line 177 "rcfile.y"
+#line 179 "rcfile.y"
 {
 		   rc_yyval.stmtlist.head = rc_yyval.stmtlist.tail = rc_yyvsp[0].stmt;
 	   ;
     break;}
 case 13:
-#line 181 "rcfile.y"
+#line 183 "rcfile.y"
 {
 		   if (rc_yyvsp[0].stmt) {
 			   if (rc_yyval.stmtlist.head == NULL) {
@@ -971,13 +973,13 @@ case 13:
 	   ;
     break;}
 case 14:
-#line 194 "rcfile.y"
+#line 196 "rcfile.y"
 {
 		   rc_yyval.stmt = NULL;
 	   ;
     break;}
 case 20:
-#line 203 "rcfile.y"
+#line 205 "rcfile.y"
 {
 		   lex_clear_state();
 		   rc_yyerrok;
@@ -986,7 +988,7 @@ case 20:
 	   ;
     break;}
 case 21:
-#line 212 "rcfile.y"
+#line 214 "rcfile.y"
 {
 		   if (!check_kw(rc_yyvsp[-1].string)) {
 			   parse_error(_("unknown keyword: %s"), rc_yyvsp[-1].string);
@@ -1004,27 +1006,27 @@ case 21:
 	   ;
     break;}
 case 22:
-#line 230 "rcfile.y"
+#line 232 "rcfile.y"
 {
 		   verbatim();
 	   ;
     break;}
 case 23:
-#line 236 "rcfile.y"
+#line 238 "rcfile.y"
 {
 		   rc_yyval.list = list_create();
 		   list_append(rc_yyval.list, rc_yyvsp[0].string);
 	   ;
     break;}
 case 24:
-#line 241 "rcfile.y"
+#line 243 "rcfile.y"
 {
 		   list_append(rc_yyvsp[-1].list, rc_yyvsp[0].string);
 		   rc_yyval.list = rc_yyvsp[-1].list;
 	   ;
     break;}
 case 26:
-#line 251 "rcfile.y"
+#line 253 "rcfile.y"
 {
 		   rc_yyval.stmt = rc_stmt_create(rc_stmt_cond);
 		   rc_yyval.stmt->v.cond.node = rc_yyvsp[-2].node;
@@ -1033,7 +1035,7 @@ case 26:
 	   ;
     break;}
 case 27:
-#line 258 "rcfile.y"
+#line 260 "rcfile.y"
 {
 		   rc_yyval.stmt = rc_stmt_create(rc_stmt_cond);
 		   rc_yyval.stmt->v.cond.node = rc_yyvsp[-4].node;
@@ -1042,13 +1044,13 @@ case 27:
 	   ;
     break;}
 case 29:
-#line 268 "rcfile.y"
+#line 270 "rcfile.y"
 {
 		   rc_yyval.node = rc_yyvsp[-1].node;
 	   ;
     break;}
 case 30:
-#line 272 "rcfile.y"
+#line 274 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_bool);
 		   rc_yyval.node->v.bool.op = bool_and;
@@ -1057,7 +1059,7 @@ case 30:
 	   ;
     break;}
 case 31:
-#line 279 "rcfile.y"
+#line 281 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_bool);
 		   rc_yyval.node->v.bool.op = bool_or;
@@ -1066,7 +1068,7 @@ case 31:
 	   ;
     break;}
 case 32:
-#line 286 "rcfile.y"
+#line 288 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_bool);
 		   rc_yyval.node->v.bool.op = bool_not;
@@ -1075,7 +1077,7 @@ case 32:
 	   ;
     break;}
 case 35:
-#line 299 "rcfile.y"
+#line 301 "rcfile.y"
 {
 		   rc_yyval.msgpart.part = HEADER;
 		   rc_yyval.msgpart.key = rc_yyvsp[0].regex;
@@ -1083,7 +1085,7 @@ case 35:
 	   ;
     break;}
 case 36:
-#line 305 "rcfile.y"
+#line 307 "rcfile.y"
 {
 		   rc_yyval.msgpart.part = HEADER;
 		   rc_yyval.msgpart.key = NULL;
@@ -1091,21 +1093,21 @@ case 36:
 	   ;
     break;}
 case 37:
-#line 313 "rcfile.y"
+#line 315 "rcfile.y"
 {
 		   rc_yyval.msgpart.string = NULL;
 		   rc_yyval.msgpart.key = NULL;
            ;
     break;}
 case 39:
-#line 321 "rcfile.y"
+#line 323 "rcfile.y"
 {
 		   rc_yyval.msgpart = rc_yyvsp[0].msgpart;
 		   rc_yyval.msgpart.part = rc_yyvsp[-1].num;
 	   ;
     break;}
 case 41:
-#line 329 "rcfile.y"
+#line 331 "rcfile.y"
 {
 		   rc_yyval.msgpart = rc_yyvsp[0].msgpart;
 		   if (rc_yyval.msgpart.key)
@@ -1113,7 +1115,7 @@ case 41:
 	   ;
     break;}
 case 42:
-#line 337 "rcfile.y"
+#line 339 "rcfile.y"
 {
 		   rc_yyval.msgpart = rc_yyvsp[0].msgpart;
 		   if (!rc_yyval.msgpart.key) {
@@ -1123,26 +1125,26 @@ case 42:
 	   ;
     break;}
 case 43:
-#line 347 "rcfile.y"
+#line 349 "rcfile.y"
 {
 		   rc_yyval.regex = anubis_regex_compile(rc_yyvsp[-1].string, rc_yyvsp[-3].num);
 		   free(rc_yyvsp[-1].string);
 	   ;
     break;}
 case 44:
-#line 354 "rcfile.y"
+#line 356 "rcfile.y"
 {
 		   rc_yyval.string = NULL;
 	   ;
     break;}
 case 45:
-#line 358 "rcfile.y"
+#line 360 "rcfile.y"
 {
 		   rc_yyval.string = rc_yyvsp[-1].string;
 	   ;
     break;}
 case 46:
-#line 364 "rcfile.y"
+#line 366 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_expr);
 		   rc_yyval.node->v.expr.part = rc_yyvsp[-4].msgpart.part;
@@ -1152,7 +1154,7 @@ case 46:
 	   ;
     break;}
 case 47:
-#line 374 "rcfile.y"
+#line 376 "rcfile.y"
 {
 		   rc_yyval.num = def_regex_modifier;
 		   reg_modifier_add(&rc_yyval.num, rc_yyvsp[0].string);
@@ -1160,7 +1162,7 @@ case 47:
 	   ;
     break;}
 case 48:
-#line 380 "rcfile.y"
+#line 382 "rcfile.y"
 {
 		   reg_modifier_add(&rc_yyvsp[-1].num, rc_yyvsp[0].string);
 		   xfree(rc_yyvsp[0].string);
@@ -1168,26 +1170,26 @@ case 48:
 	   ;
     break;}
 case 49:
-#line 388 "rcfile.y"
+#line 390 "rcfile.y"
 {
 		   rc_yyval.num = def_regex_modifier;
 	   ;
     break;}
 case 51:
-#line 395 "rcfile.y"
+#line 397 "rcfile.y"
 {
 		   rc_yyval.string = rc_yyvsp[0].string;
 	   ;
     break;}
 case 52:
-#line 401 "rcfile.y"
+#line 403 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
 	   ;
     break;}
 case 55:
-#line 414 "rcfile.y"
+#line 416 "rcfile.y"
 {
 		   rc_yyval.stmt = rc_stmt_create(rc_stmt_rule);
 		   rc_yyval.stmt->v.rule.node = rc_yyvsp[-3].node;
@@ -1195,7 +1197,7 @@ case 55:
 	   ;
     break;}
 case 56:
-#line 422 "rcfile.y"
+#line 424 "rcfile.y"
 {
 		   rc_yyval.node = rc_node_create(rc_node_expr);
 		   rc_yyval.node->v.expr.part = HEADER;
@@ -1205,14 +1207,14 @@ case 56:
 	   ;
     break;}
 case 57:
-#line 432 "rcfile.y"
+#line 434 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
 	   ;
     break;}
 case 60:
-#line 443 "rcfile.y"
+#line 445 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1225,7 +1227,7 @@ case 60:
 	   ;
     break;}
 case 61:
-#line 454 "rcfile.y"
+#line 456 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1238,7 +1240,7 @@ case 61:
 	   ;
     break;}
 case 62:
-#line 465 "rcfile.y"
+#line 467 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1251,7 +1253,7 @@ case 62:
 	   ;
     break;}
 case 63:
-#line 476 "rcfile.y"
+#line 478 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1264,7 +1266,7 @@ case 63:
 	   ;
     break;}
 case 64:
-#line 487 "rcfile.y"
+#line 489 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1277,7 +1279,7 @@ case 64:
 	   ;
     break;}
 case 65:
-#line 498 "rcfile.y"
+#line 500 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1293,7 +1295,7 @@ case 65:
 	   ;
     break;}
 case 66:
-#line 514 "rcfile.y"
+#line 516 "rcfile.y"
 {
 		   if (!is_prog_allowed())
 			   YYERROR;
@@ -1523,7 +1525,7 @@ rc_yyerrhandle:
     }
   return 1;
 }
-#line 522 "rcfile.y"
+#line 524 "rcfile.y"
 
 
 static int
@@ -2141,6 +2143,7 @@ struct eval_env {
 	char **refstr;
 	jmp_buf jmp;
 	RC_LOC loc;
+	int traceable;
 };
 
 static void asgn_eval(struct eval_env *env, RC_ASGN *asgn);
@@ -2226,7 +2229,8 @@ asgn_eval(struct eval_env *env, RC_ASGN *asgn)
 	if (!p)
 		return;
 
-	tracefile(&env->loc, _("Executing %s"), asgn->lhs);
+	if (env->traceable)
+		tracefile(&env->loc, _("Executing %s"), asgn->lhs);
 	if (env->refstr) {
 		char *s;
 		LIST *arg = list_create();
@@ -2295,11 +2299,17 @@ expr_eval(struct eval_env *env, RC_EXPR *expr)
 	default:
 		abort();
 	}
-	if (rc)
-		tracefile(&env->loc, _("Matched condition %s[%s] \"%s\""),
-		      part_string(expr->part),
-		      VALID_STR(expr->key),
-		      anubis_regex_source(expr->re));
+	if (rc) {
+		if (strcmp(VALID_STR(expr->key), X_ANUBIS_RULE_HEADER) == 0)
+			tracefile(&env->loc, _("Matched trigger \"%s\""),
+				  anubis_regex_source(expr->re));
+		else
+			tracefile(&env->loc, 
+				  _("Matched condition %s[%s] \"%s\""),
+		      		  part_string(expr->part),
+		      		  VALID_STR(expr->key),
+		      		  anubis_regex_source(expr->re));
+	}
 	return rc;
 }
 
@@ -2401,8 +2411,10 @@ eval_section(int method, RC_SECTION *sec, struct rc_secdef *secdef,
 	env.msg = msg;
 	env.data = data;
 	env.loc = sec->loc;
+	env.traceable = secdef->allow_prog;
 	
-	tracefile(&sec->loc, "Section %s", sec->name);
+	if (env.traceable)
+		tracefile(&sec->loc, "Section %s", sec->name);
 	
 	if (setjmp(env.jmp) == 0)
 		stmt_list_eval(&env, sec->stmt);
