@@ -13,6 +13,11 @@ typedef union {
 		RC_REGEX *key;
 		char *string;
 	} msgpart;
+	RC_LOC loc;
+	struct {
+		size_t line;
+		char *name;
+	} begin_sec;
 	LIST *list;
 } YYSTYPE;
 #define	EOL	257
